CON
	SYNC_W			= 6*16
	VIDEO_W			= 320
	VIDEO_H			= 480
	CHAR_W			= 40
	CHAR_H			= 20
	FONT_W			= 8
	FONT_H			= 12
	FRONT_PORCH		= 31
	VSYNC_ROWS		= 2
	BOTTOM_PORCH		= 11

	video_pll16x		= 15

VAR
	long	cog

PUB start(sync_mem, pixel_mem, color_mem, vga_pin) : okay
	stop
	sync_ptr := sync_mem
	pixmem_ptr := pixel_mem
	colmem_ptr := color_mem
	okay := cog := cognew(@entry, vga_pin)

PUB stop
	if cog
		cogstop(cog~ - 1)

DAT
		org	0
entry
		mov	vpin, PAR			' pin #
		mov	xdira, #$ff			' 8 pins
		shl	xdira, vpin			' shift to pin group position

		shr	vpin, #3			' convert pin number into pin group
		'               vmode=01   cmode=0  chroma1  chroma0  auralsub
		mov	xvcfg, #(%01<<6) | (1<<5) | (0<<4) | (0<<3) | %000
		shl	xvcfg, #(23 - 9)
		or	xvcfg, vpin			' or with video pin group
		shl	xvcfg, #9
		or	xvcfg, #$ff			' all 8 pins

		mov	DIRA, xdira			' make the 8 video pins outputs
		mov	OUTA, #0			' and set all outputs to 0

		movi	CTRA, #video_pll16x		' Setup for counter output
		movd	CTRA, #0
		movs	CTRA, #0
		mov	FRQA, xfrqa
		mov	VSCL, xvscl			' Setup video scale

		mov	VCFG, xvcfg
frame
		mov	pixptr, pixmem_ptr
		mov	colptr, colmem_ptr
		mov	char_rows, #CHAR_H		' 40 character rows
		mov	scanline, #2
:char_row
		mov	scan_rows, #FONT_H		' 12 scanlines per character
:scan_row
		waitvid	H_SYNC, #0
		waitvid	H_SYNC, #0
		waitvid	H_SYNC, #0
		waitvid	H_PORCH, #0			' should be 1.89us = 48 pixels
		mov	cols, #CHAR_W / 2		' number of 16 pixel frames
		waitvid	H_PORCH, #0
:col1
		rdlong	colors, colptr
		add	colptr, #4
		rdlong	pixels, pixptr
		add	pixptr, #4
		waitvid	colors, pixels
		djnz	cols, #:col1


		waitvid	H_SYNC, #0			' should be 3.77us = 95 pixels
		sub	colptr, #CHAR_W	* 2		' use the same palette row again
		waitvid	H_SYNC, #0
		sub	pixptr, #CHAR_W * 2
		waitvid	H_SYNC, #0
		cmp	scan_rows, #1		WZ
		waitvid	H_PORCH, #0			' should be 1.89us = 48 pixels
	if_z	wrlong	scanline, sync_ptr		' just did char row #0: send sync
		waitvid	H_PORCH, #0
		mov	cols, #CHAR_W / 2		' number of 16 pixel frames
:col2
		rdlong	colors, colptr
		add	colptr, #4
		rdlong	pixels, pixptr
		add	pixptr, #4
		waitvid	colors, pixels
		djnz	cols, #:col2

		sub	colptr, #CHAR_W	* 2		' use the same palette row again
		djnz	scan_rows, #:scan_row

		add	colptr, #CHAR_W	* 2		' next palette row 
		cmp	pixptr, colmem_ptr	WZ
	if_z	mov	pixptr, pixmem_ptr		' after char row #2: wrap pixptr
	if_z	mov	colptr, colmem_ptr		' and also colptr
		add	scanline, #1
		djnz	char_rows, #:char_row		' next character row

		mov	rows, #BOTTOM_PORCH		' bottom porch rows
:vblank1_row
		waitvid	H_SYNC, #0
		waitvid	H_SYNC, #0
		waitvid	H_SYNC, #0
		cmp	rows, #1		WZ
		waitvid	H_PORCH, #0
	if_z	wrlong	scanline, sync_ptr
		waitvid	H_PORCH, #0
		mov	cols, #CHAR_W / 2		' blank frames
:vblank1_col	waitvid	H_PORCH, #0			' for VIDEO_W pixels time
		djnz	cols,#:vblank1_col
		djnz	rows,#:vblank1_row

		mov	rows, #VSYNC_ROWS		' vertical sync rows
		mov	scanline, #0
:vsync_row
		waitvid	V_AND_H_SYNC, #0
		waitvid	V_AND_H_SYNC, #0
		waitvid	V_AND_H_SYNC, #0
		cmp	rows, #1		WZ
		waitvid	V_PORCH, #0
	if_z	wrlong	scanline, sync_ptr
		waitvid	V_PORCH, #0
		mov	cols, #CHAR_W / 2		' blank frames with VSYNC low
:vsync_col	waitvid	V_PORCH, #0			' for VIDEO_W pixels time
		djnz	cols, #:vsync_col
		djnz	rows, #:vsync_row

		mov	rows, #FRONT_PORCH		' front porch rows
		mov	scanline, #0
:vblank2_row
		waitvid	H_SYNC, #0
		waitvid	H_SYNC, #0
		waitvid	H_SYNC, #0
		cmp	rows, #FRONT_PORCH / 2		WZ
		waitvid	H_PORCH, #0
	if_z	wrlong	scanline, sync_ptr
		waitvid	H_PORCH, #0
		mov	cols, #CHAR_W / 2		' blank frames
:vblank2_col	waitvid	H_PORCH, #0			' for VIDEO_W pixels time
		djnz	cols,#:vblank2_col
		djnz	rows,#:vblank2_row

		jmp	#frame				' next frame

'**********************************************************************
xdira		long	0
xvcfg		long	0
xvscl		long	(10 << 12) | (10 * 16)			' 3 PLL clocks per pixel, frames of 16 pixels
xfrqa		long	24 << 24				' 125 MHz => 20MHz pixel clock

sync_ptr	long	0-0					' pointer to the cog sync long
pixmem_ptr	long	0-0					' pointer to the pixel memory
colmem_ptr	long	0-0					' pointer to the palette memory

H_PORCH		long	$03030303				' both sync signals high, color #0
V_PORCH		long	$02020202				' vertical sync signal low, color #0
H_SYNC		long	$01010101				' horizontal sync signal low, color #0
V_AND_H_SYNC	long	$00000000				' both sync signals low, color #0

scanline	long	0
minus1		long	-1

vpin		res	1
rows		res	1
cols		res	1
char_rows	res	1
scan_rows	res	1
pixptr		res	1
pixels		res	1
colptr		res	1
colors		res	1
t1		res	1

		fit	$1f0
